BEGIN { $INC{'Win32/GUI/Window.pm'} = 1; }
#perl2exe_include "Win32/GUI/Loft/Control/ForegroundBackground.pm"
#perl2exe_include "Win32/GUI/Loft/Control/Intangible.pm"
#perl2exe_include "Win32/GUI/Loft/Control/Button.pm"
#perl2exe_include "Win32/GUI/Loft/Control/Checkbox.pm"
#perl2exe_include "Win32/GUI/Loft/Control/Combobox.pm"
#perl2exe_include "Win32/GUI/Loft/Control/Custom.pm"
#perl2exe_include "Win32/GUI/Loft/Control/Graphic.pm"
#perl2exe_include "Win32/GUI/Loft/Control/Groupbox.pm"
#perl2exe_include "Win32/GUI/Loft/Control/Hyperlink.pm"
#perl2exe_include "Win32/GUI/Loft/Control/ImageList.pm"
#perl2exe_include "Win32/GUI/Loft/Control/Label.pm"
#perl2exe_include "Win32/GUI/Loft/Control/Listbox.pm"
#perl2exe_include "Win32/GUI/Loft/Control/Listview.pm"
#perl2exe_include "Win32/GUI/Loft/Control/ProgressBar.pm"
#perl2exe_include "Win32/GUI/Loft/Control/Radiobutton.pm"
#perl2exe_include "Win32/GUI/Loft/Control/RichEdit.pm"
#perl2exe_include "Win32/GUI/Loft/Control/Splitter.pm"
#perl2exe_include "Win32/GUI/Loft/Control/StatusBar.pm"
#perl2exe_include "Win32/GUI/Loft/Control/TabStrip.pm"
#perl2exe_include "Win32/GUI/Loft/Control/Textfield.pm"
#perl2exe_include "Win32/GUI/Loft/Control/Timer.pm"
#perl2exe_include "Win32/GUI/Loft/Control/Toolbar.pm"
#perl2exe_include "Win32/GUI/Loft/Control/TreeView.pm"
#perl2exe_include "Win32/GUI/Loft/Control/Window.pm"

# NTray - A tray notification tool for the Nagios Monitoring System
# Copyright (C) 2002 Robert M. Wagner (rob@robandsara.com)
# Licensed under the GPL (see included LICENSE file for full license)
# 
# Nagios is a registered trademark of Ethan Galstad. (nagios@nagios.org)
# You can visit the Nagios home page at http://www.nagios.org
#
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
no strict "refs";


use bytes;
use Win32::GUI;
use Win32 ();
use Win32::API;
use Win32::FileOp;
use Win32::GUI::Loft::Design;
use Win32::GUI::Loft::Control;
use sigtrap;

use XML::Simple;
use XML::Parser::Style::Tree;
use utf8;
#perl2exe_include utf8;
#perl2exe_include "unicore/lib/Word.pl";
#perl2exe_include "unicore/lib/Digit.pl";
#perl2exe_include "unicore/lib/SpacePer.pl";
#perl2exe_include "unicore/To/Lower.pl";
use LWP::UserAgent;
use URI::https; #perl2exe
use LWP::Protocol::https; #perl2exe
use Net::SSL; #perl2exe
use HTML::TokeParser::Simple;
use Win32::Sound;


my $version = "v1.0b"; #Just for reference :)


#Load GUI Loft designer files
my $fileSettings = "ntray_settingswindow.gld";
my $fileStatus = "ntray_statuswindow.gld";

my $objSettings = Win32::GUI::Loft::Design->newLoad($fileSettings) or
                        die("Could not open window file ($fileSettings)");

my $objStatus = Win32::GUI::Loft::Design->newLoad($fileStatus) or
                        die("Could not open window file ($fileStatus)");

my $winSettings = $objSettings->buildWindow() or die("Could not build window ($fileSettings)");
my $winStatus = $objStatus->buildWindow() or die("Could not build window ($fileStatus)");



#Needed some globals
my ($previous_color,$xml,$config,$green,$yellow,$red,$unknown,$logo);
$green = Win32::GUI::Icon->new('green.ico');
$yellow = Win32::GUI::Icon->new('yellow.ico');
$red = Win32::GUI::Icon->new('red.ico');
$unknown = Win32::GUI::Icon->new('unknown.ico');
$logo = Win32::GUI::Icon->new('logo.ico');
my (@serviceproblems,@hostproblems);
my ($status_color,$status_text,$previous_text);
my $icoTrayicon;
my $icoMenu;

my $configfile;

main();




sub main {
   #Initialization
   my ($override_configfile,$override_refresh) = parse_command_line();
   if (! check_icons()) {
      Win32::GUI::MessageBox(0, "Application Error: Some or all icons are missing!\nMake sure red.ico, green.ico, yellow.ico, \nand unknown.ico exist in the current directory.", "NTray Error");
      die ("Application Error: Some or all icons are missing!\nMake sure red.ico, green.ico, yellow.ico, \nand unknown.ico exist in the current directory.");
   }
   
   my ($check_configfile,$configfile) = check_configfile($override_configfile);
   if (! $check_configfile) {
      #Replace error here with dialogbox to prompt if user wants to configure NTray or not.
      Win32::GUI::MessageBox(0, "Application Error: Configuration file not found! ($configfile)", "NTray Error");
      die ("Application Error: Configuration file not found! ($configfile)");
   }




   #Read in the config file, since we've already tested for it's existence.
   $xml = XML::Simple->new();
   $config = $xml->XMLin("$configfile", keeproot => 1);   

   #'Decrypt' the password fields in the config file
   $config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD} = join("",   map { chr } (split /\\/,$config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD} ));
   $config->{CONFIG}->{PROXY}->{PROXY_PASSWORD} = join("",   map { chr } (split /\\/,$config->{CONFIG}->{PROXY}->{PROXY_PASSWORD}));

   my $DesktopWin = $winStatus->GetDesktopWindow();
   my (undef,undef,$DesktopWidth,$DesktopHeight) = Win32::GUI::GetWindowRect($DesktopWin);
   
   if ($config->{CONFIG}->{POPUPS}->{POPUP_XPOS} > ($DesktopWidth - 100)) {
      $config->{CONFIG}->{POPUPS}->{POPUP_XPOS} = 100;
   }

   if ($config->{CONFIG}->{POPUPS}->{POPUP_YPOS} > ($DesktopHeight - 100)) {
      $config->{CONFIG}->{POPUPS}->{POPUP_YPOS} = 100;
   }


   #Reset last known Window positions
   $winStatus->Left($config->{CONFIG}->{POPUPS}->{POPUP_XPOS});
   $winStatus->Top($config->{CONFIG}->{POPUPS}->{POPUP_YPOS});
   
     
   my $refresh = $config->{CONFIG}->{BASIC}->{REFRESH_TIMER} * 1000;
   print "Refresh is: $refresh\n";
   $Win32::GUI::Loft::window{winStatus}->timStatusTimer->Interval($refresh);

   #Disable auto-dismiss timer.
   $Win32::GUI::Loft::window{winStatus}->timDismissTimer->Kill();

   
   #Get initial status from webserver (other object creates are dependent on this)
   ($status_color,$status_text) = getStatus();

   print "Status Color is $status_color\n";
   $previous_color = $status_color;
   $previous_text = $status_text;
   
   #Create Initial Icon
   create_icoTrayicon($status_color,$status_text);

   #Pretty things up
   $winSettings->ChangeIcon($logo);
   $winStatus->ChangeIcon($logo);

   
   #And finally.. start the dialog loop
   Win32::GUI::Dialog();

   #On termination, write config
   write_configfile("$configfile");
}


#Subroutines
sub parse_command_line {
   my @commandline = @ARGV;
   my $override_configfile = "None";
   my $override_refresh = 0;


   while (scalar(@commandline) > 0) {
      my $command = shift(@commandline);
  
      if ($command =~ /--help/i) {
         Win32::GUI::MessageBox(0, "NTray $version Usage:\nNTray.exe \[-c d:\\path\\config.xml\]\n\nExample:\nNTray.exe -c c:\\etc\\ntrayconfig.xml\n", "NTray Help");
         die ("NTray $version Usage:\nNTray.exe \[-c d:\\path\\config.xml\]\n\nExample:\nNTray.exe -c c:\\etc\\ntrayconfig.xml\n");
      }
      
      if ($command =~ /-c/) {
         my $override_configfile = shift(@commandline);
         print "Overriding config file default. Using $override_configfile\n";
      }

   }
   return ($override_configfile,$override_refresh);
}

sub check_configfile {
   my ($override_configfile) = @_;
   $configfile = "./config.xml";

   if ($override_configfile ne "None") {
      $configfile = $override_configfile;
   }

   if (! -e "$configfile") {
      #Prompt - Config file not found - Do you want to configure?
      return (0,$configfile);
   }
   return (1,$configfile);
}

sub check_icons {
   #Verify that all required icons exist and can be used
   if ((-e "green.ico") && (-e "yellow.ico") && (-e "red.ico") && (-e "unknown.ico")) {
      return 1;
   } else {
      return 0;
   }
}

sub create_icoTrayicon {
   my ($status_color,$status_text) = @_;
   if ($status_color eq "red") {
      $icoTrayicon = $winStatus->AddNotifyIcon(-name => "icoTrayicon",
                                               -tip  => "$status_text",
                                               -icon => $red,
                                               -id   => $$ ); #Set the ID equal to the PID of this process
   }
   if ($status_color eq "yellow") {
      $icoTrayicon = $winStatus->AddNotifyIcon(-name => "icoTrayicon",
                                               -tip  => "$status_text",
                                               -icon => $yellow,
                                               -id   => $$ ); #Set the ID equal to the PID of this process
   }
   if ($status_color eq "green") {
      $icoTrayicon = $winStatus->AddNotifyIcon(-name => "icoTrayicon",
                                               -tip  => "$status_text",
                                               -icon => $green,
                                               -id   => $$ ); #Set the ID equal to the PID of this process
   }
   if ($status_color eq "unknown") {
      $icoTrayicon = $winStatus->AddNotifyIcon(-name => "icoTrayicon",
                                               -tip  => "$status_text",
                                               -icon => $unknown,
                                               -id   => $$ ); #Set the ID equal to the PID of this process
   }
   print "Icon Create: Color is $status_color\nID is $$\n";
   1,
}

sub change_icon {
   my ($status_color,$status_text) = @_;

   if ($status_color eq "red") {
      Win32::GUI::NotifyIcon::Modify($winStatus,
                                     -tip  => "$status_text",
                                     -icon => $red,
                                     -id   => $$ ); #Set the ID equal to the PID of this process
   }
   if ($status_color eq "yellow") {
      Win32::GUI::NotifyIcon::Modify($winStatus,
                                     -tip  => "$status_text",
                                     -icon => $yellow,
                                     -id   => $$ ); #Set the ID equal to the PID of this process
   }
   if ($status_color eq "green") {
      Win32::GUI::NotifyIcon::Modify($winStatus,
                                     -tip  => "$status_text",
                                     -icon => $green,
                                     -id   => $$ ); #Set the ID equal to the PID of this process
   }
   if ($status_color eq "unknown") {
      Win32::GUI::NotifyIcon::Modify($winStatus,
                                     -tip  => "$status_text",
                                     -icon => $unknown,
                                     -id   => $$ ); #Set the ID equal to the PID of this process
   }
   print "Icon Update: Color is $status_color\nID is $$\n";


   1; #Don't end the dialog() loop
}


sub getStatus {
   my $color = "unknown";
   my ($counthostproblems,$countserviceproblems) = (0,0);

   my $ua = LWP::UserAgent->new;

   my $url; 
   
   if ($config->{CONFIG}->{BASIC}->{UNHANDLED_ONLY} =~ /true/i) {
      $url = $config->{CONFIG}->{BASIC}->{CGI_URL} . "/statuswml.cgi?style=uprobs";
   } else {
      $url = $config->{CONFIG}->{BASIC}->{CGI_URL} . "/statuswml.cgi?style=aprobs";
   }


   my $req = HTTP::Request->new(GET => $url);

   if ($config->{CONFIG}->{PROXY}->{PROXY_REQUIRED} =~ /true/i) {
      my $proxyaddress = "http://$config->{CONFIG}->{PROXY}->{PROXY_SERVER}:$config->{CONFIG}->{PROXY}->{PROXY_PORT}";
      $ua->proxy('http' => $proxyaddress);
      if ($config->{CONFIG}->{PROXY}->{AUTH_REQUIRED} =~ /true/i) {
         $req->proxy_authorization_basic($config->{CONFIG}->{PROXY}->{PROXY_USERNAME},$config->{CONFIG}->{PROXY}->{PROXY_PASSWORD} );
      }
   }
   
   if ($config->{CONFIG}->{AUTHENTICATION}->{AUTH_REQUIRED} =~ /true/i) {
      $req->authorization_basic($config->{CONFIG}->{AUTHENTICATION}->{AUTH_USERNAME},$config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD});
   }

   my $content = $ua->request($req)->as_string;

   my $p = HTML::TokeParser::Simple->new(\$content);
   
   #What section of the HTML are we in?
   my $section = "header";
   my $hostdetail = 0;
   my $servicedetail = 0;
   my (@hostdetail,@servicedetail);
   my ($status,$host,$service);
   
   $#hostproblems = -1;
   $#serviceproblems = -1;
   
   while ( my $token = $p->get_token ) {
      $token->rewrite_tag;
      my $tokendata = $token->as_is;
   
      if (($tokendata eq "") || ($tokendata eq "\n")) {
         next;
      }
   
      if (!$token->is_text) {
         next;
      }
   
      #print "$tokendata\n";
      if ($tokendata =~ "Host") {
         $section = "host";
         $color = "green";
         next;
      }
   
      if ($tokendata =~ "Svc") {
         $section = "service";
         next;
      }
   
      if ($section eq "host") {
         if ($tokendata eq "No problems") {
            next;
         }
         $hostdetail[$hostdetail] = $tokendata;
         if ($hostdetail == 0) {
            ++$hostdetail;
            next;
         } else {
            $status = $hostdetail[0];
            if ((($status eq "DWN") || ($status eq "UNR")) && ($color eq "green")) {
               $color = "red";
            }
            $host = $hostdetail[1];
            push @hostproblems,($host,$status);
            print "HOST PROBLEM: Host: $host\tStatus: $status\n";
            ++$counthostproblems;
            $hostdetail = 0;
            next;
         }
      }
   
      if ($section eq "service") {
         if ($tokendata eq "No problems") {
            next;
         }
         $servicedetail[$servicedetail] = $tokendata;
         if ($servicedetail == 0) {
            ++$servicedetail;
            next;
         } else {
            $status = $servicedetail[0];
            if (($status eq "CRI") && (($color eq "green") || ($color eq "yellow"))) {
               $color = "red";
            }
            if (($status eq "WRN") && ($color eq "green")) {
                $color = "yellow";
            }
            ($host,$service) = split(/\//,$servicedetail[1]);
            print "SERVICE PROBLEM: Host: $host\tService:$service\tStatus\t$status\n";
            push @serviceproblems,($host,$service,$status);
            ++$countserviceproblems;
            $servicedetail = 0;
            next;
         }
      }
   }
   $status_text = "Host: $counthostproblems  Svc: $countserviceproblems";
   update_lvwStatus($counthostproblems,$countserviceproblems);
   return ($color,$status_text);
}

sub update_lvwStatus {
   my ($counthostproblems,$countserviceproblems) = @_;
   my $listviewcount = $winStatus->lvwStatus->Count();
   my $x = 0;  
   my $y = 0;
   $winStatus->lvwStatus->Clear;
   for ($x = 0;$x < $counthostproblems;$x++) {
      my $host = $hostproblems[($x * 2)];
      my $hoststatus = $hostproblems[(($x * 2) + 1)];
      print "Populating Host $host $hoststatus\n";
      if ($hoststatus eq "DWN") {
         $hoststatus = "Down";
      }
      if ($hoststatus eq "UNR") {
         $hoststatus = "Unreachable";
      }


      if ($x == 0) {
         $winStatus->lvwStatus->InsertItem(-selected => 1,
                             -text => "H",
                             );
         $winStatus->lvwStatus->SetItem(-item => $x,
                                        -subitem => 1,
                                        -text => "$host");

         $winStatus->lvwStatus->SetItem(-item => $x,
                                        -subitem => 2,
                                        -text => "HOST");

         $winStatus->lvwStatus->SetItem(-item => $x,
                                        -subitem => 3,
                                        -text => "$hoststatus");
      } else {
         $winStatus->lvwStatus->InsertItem(-text => "H");
         $winStatus->lvwStatus->SetItem(-item => $x,
                                        -subitem => 1,
                                        -text => "$host");

         $winStatus->lvwStatus->SetItem(-item => $x,
                                        -subitem => 2,
                                        -text => "HOST");

         $winStatus->lvwStatus->SetItem(-item => $x,
                                        -subitem => 3,
                                        -text => "$hoststatus");
      }
   }
   for ($y = 0;$y < $countserviceproblems;$y++) {
      my $host = $serviceproblems[($y * 3)];
      my $service = $serviceproblems[(($y * 3) + 1)];
      my $servicestatus = $serviceproblems[(($y * 3) + 2)];
      if ($servicestatus eq "CRI") {
         $servicestatus = "Critical";
      }
      if ($servicestatus eq "WRN") {
         $servicestatus = "Warning";
      }

      my $itemnumber = $x + $y;

      if (($y == 0) && ($counthostproblems == 0)) {
         $winStatus->lvwStatus->InsertItem(-selected => 1,
                             -text => "S",
                             );
         $winStatus->lvwStatus->SetItem(-item => $itemnumber,
                                        -subitem => 1,
                                        -text => "$host");

         $winStatus->lvwStatus->SetItem(-item => $itemnumber,
                                        -subitem => 2,
                                        -text => "$service");

         $winStatus->lvwStatus->SetItem(-item => $itemnumber,
                                        -subitem => 3,
                                        -text => "$servicestatus");
      } else {
         $winStatus->lvwStatus->InsertItem(-text => "S");
         $winStatus->lvwStatus->SetItem(-item => $itemnumber,
                                        -subitem => 1,
                                        -text => "$host");

         $winStatus->lvwStatus->SetItem(-item => $itemnumber,
                                        -subitem => 2,
                                        -text => "$service");

         $winStatus->lvwStatus->SetItem(-item => $itemnumber,
                                        -subitem => 3,
                                        -text => "$servicestatus");
      }
   }



}


sub timStatusTimer_Timer {
   #This is executed at $refresh seconds.
   my ($status_color,$status_text) = getStatus();

   if (($config->{CONFIG}->{SOUND}->{ENABLE_SOUND} =~ /true/i) && ($status_color ne $previous_color)) {
      if ((($config->{CONFIG}->{SOUND}->{SOUND_CRITONLY} =~ /true/i) && ($status_color eq "red")) || ($config->{CONFIG}->{SOUND}->{SOUND_CRITONLY} =~ /false/i)) {
         play_sound($status_color);
      }
   }

   if ($status_color ne $previous_color) {   
      change_icon($status_color,$status_text);
      if ($config->{CONFIG}->{POPUPS}->{ENABLE_POPUPS} =~ /true/i) {
         if (($config->{CONFIG}->{POPUPS}->{POPUPS_CRITONLY} =~ /true/i) && ($status_color eq "red")) {
            show_popup($status_color,$status_text);
         } elsif ($config->{CONFIG}->{POPUPS}->{POPUPS_CRITONLY} =~ /false/i) {
            show_popup($status_color,$status_text);
         }
      }
   }

   $previous_color = $status_color;
   $previous_text = $status_text;
}


sub show_popup {
   my ($status_color,$status_text,$dont_dismiss) = @_;
   #Set auto-dismiss timer on Status window if necessary
   if (($config->{CONFIG}->{POPUPS}->{AUTO_DISMISS} =~ /true/i) && (! $dont_dismiss)) {
      my $dismiss_time = $config->{CONFIG}->{POPUPS}->{DISMISS_TIMER} * 1000;
      print "Window will self-destruct in $dismiss_time milliseconds\n";
      $Win32::GUI::Loft::window{winStatus}->timDismissTimer->Interval($dismiss_time);
   }  

   $winStatus->Hide();
   #Change icon color in corner  - This code doesn't work for some reason with the LOFT gui. Commented out for now.
#   if ($status_color eq "green") {
#      $winStatus->ChangeIcon($green);
#   }
#   if ($status_color eq "yellow") {
#
#   }
#   if ($status_color eq "red") {
#      $winStatus->ChangeIcon($red);
#   }
#   if ($status_color eq "unknown") {
#      $winStatus->ChangeIcon($unknown);
#   }

   $winStatus->Show();
   $winStatus->BringWindowToTop();
}

sub play_sound {
   my ($status_color) = @_;
   my $soundfile = "$status_color".".wav";
   if (-e $soundfile) {
      Win32::Sound::Play($soundfile, SND_ASYNC);
      #Win32::Sound::Stop();
   }
}

sub timDismissTimer_Timer {
      $Win32::GUI::Loft::window{winStatus}->timDismissTimer->Kill();
   print "Auto-hiding winStatus\n";
   winStatus_Terminate();
}

sub winStatus_Terminate {
   #Store Window location for later use
   #print "Hiding winStatus\n";
   $config->{CONFIG}->{POPUPS}->{POPUP_XPOS} = $winStatus->Left();
   $config->{CONFIG}->{POPUPS}->{POPUP_YPOS} = $winStatus->Top();

   $winStatus->Hide();
   0;   
}

sub btnStatusDismiss_Click {
      $Win32::GUI::Loft::window{winStatus}->timDismissTimer->Kill();
   winStatus_Terminate();
   0;
}

sub btnStatusOpen_Click {
      $Win32::GUI::Loft::window{winStatus}->timDismissTimer->Kill();
   my ($selected_item) = $winStatus->lvwStatus->SelectedItems();
   my ($host,$service,$recordtype,$junk,$url);
   ($junk,$recordtype) = $winStatus->lvwStatus->ItemInfo($selected_item);
   ($junk,$host) = $winStatus->lvwStatus->ItemInfo($selected_item,1);
   ($junk,$service) = $winStatus->lvwStatus->ItemInfo($selected_item,2);
   
   $host = convertToURL($host);
   $service = convertToURL($service);
   
   if ($recordtype eq "H") {
      $url = "$config->{CONFIG}->{BASIC}->{CGI_URL}"."/extinfo.cgi?type=1\&host=$host";
   }
   if ($recordtype eq "S") {
      $url = "$config->{CONFIG}->{BASIC}->{CGI_URL}"."/extinfo.cgi?type=2\&host=$host\&service=$service";   
   }
   
   Win32::FileOp::ShellExecute 'Open' => "$url";
   0;   
}

sub btnStatusAck_Click {
      $Win32::GUI::Loft::window{winStatus}->timDismissTimer->Kill();
   my ($selected_item) = $winStatus->lvwStatus->SelectedItems();
   my ($host,$service,$recordtype,$junk,$url);
   ($junk,$recordtype) = $winStatus->lvwStatus->ItemInfo($selected_item);
   ($junk,$host) = $winStatus->lvwStatus->ItemInfo($selected_item,1);
   ($junk,$service) = $winStatus->lvwStatus->ItemInfo($selected_item,2);
   
   $host = convertToURL($host);
   $service = convertToURL($service);
   
   if ($recordtype eq "H") {
      $url = "$config->{CONFIG}->{BASIC}->{CGI_URL}"."/cmd.cgi?cmd_typ=33\&host=$host";
   }
   if ($recordtype eq "S") {
      $url = "$config->{CONFIG}->{BASIC}->{CGI_URL}"."/cmd.cgi?cmd_typ=34\&host=$host\&service=$service";   
   }
   
   Win32::FileOp::ShellExecute 'Open' => "$url";
   0;   
}

sub btnStatusNagios_Click {
      $Win32::GUI::Loft::window{winStatus}->timDismissTimer->Kill();
   my $url = "$config->{CONFIG}->{BASIC}->{NAGIOS_URL}";
   Win32::FileOp::ShellExecute 'Open' => "$url";
   0;
}

sub lvwStatus_Click {
      $Win32::GUI::Loft::window{winStatus}->timDismissTimer->Kill();
   0;
}

sub winStatus_Click {
      $Win32::GUI::Loft::window{winStatus}->timDismissTimer->Kill();
   0;
}

sub icoTrayicon_Click {
   show_popup($status_color,$status_text,'1');
}

sub convertToURL {
   my ($data) = @_;
   $_=$data;
   my $out;
   foreach (split //,$_){
      if ($_ eq " ") { $out.="+"; next; }    
      if((ord($_) >= 0x41 && ord($_) <= 0x5a) || (ord($_) >= 0x61 && ord($_) <= 0x7a) || (ord($_) >= 0x30 && ord($_) <= 0x39)) {
         $out.=$_ ;
      } else {
         $out.=sprintf("%%%02X",ord($_)); 
      }
   }
return $out;
}

sub icoTrayicon_RightClick {
   #Right Click on the Tray Icon
   my ($x,$y) = Win32::GUI::GetCursorPos();
   $icoMenu = new Win32::GUI::Menu(
      "tray"       => "tray",
      " > &Nagios" => "Nagios",
      " > -"      => 0,
      " > &Status" => "Status",
      " > &Refresh" => "Refresh",
      " > &Config" => "Config",
      " > &About"  => "About",
      " > E&xit"   => "Exit",
      );


   $winStatus->TrackPopupMenu($icoMenu->{tray}, $x, $y);

1; #Don't end the dialog loop
}

sub Nagios_Click {
   my $url = "$config->{CONFIG}->{BASIC}->{NAGIOS_URL}";
   Win32::FileOp::ShellExecute 'Open' => "$url";
   1
}

sub Status_Click {
   show_popup($status_color,$status_text,'1');
   1;
}

sub Refresh_Click {
   timStatusTimer_Timer();
   1;
}


sub Config_Click {
   show_config();
   
   1;
}

sub About_Click {
   #Click on the 'About' Menu Option
   Win32::GUI::MessageBox(0, "Ntray $version\n\n(c)2005 by Rob Wagner (rob\@robandsara.com)\n\nLicensed Under the GPL\nNtray PID = $$\n", "About NTray");
   1;
}

sub Exit_Click {
   -1;
}

sub show_config {
   #Show the configuration window.. repopulate values
   $Win32::GUI::Loft::window{winSettings}->tfSettingsNagiosCGIUrl->Text("$config->{CONFIG}->{BASIC}->{CGI_URL}");
   $Win32::GUI::Loft::window{winSettings}->tfSettingsNagiosURL->Text("$config->{CONFIG}->{BASIC}->{NAGIOS_URL}");
   $Win32::GUI::Loft::window{winSettings}->tfSettingsTimeRefresh->Text("$config->{CONFIG}->{BASIC}->{REFRESH_TIMER}");
   $Win32::GUI::Loft::window{winSettings}->tfSettingsPopupDismissTime->Text("$config->{CONFIG}->{POPUPS}->{DISMISS_TIMER}");


   $Win32::GUI::Loft::window{winSettings}->tfSettingsNagiosAuthUsername->Text("$config->{CONFIG}->{AUTHENTICATION}->{AUTH_USERNAME}");
   $Win32::GUI::Loft::window{winSettings}->tfSettingsNagiosAuthPassword->Text("$config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD}");
   $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyAddress->Text("$config->{CONFIG}->{PROXY}->{PROXY_SERVER}");
   $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyPort->Text("$config->{CONFIG}->{PROXY}->{PROXY_PORT}");
   $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyUsername->Text("$config->{CONFIG}->{PROXY}->{PROXY_USERNAME}");
   $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyPassword->Text("$config->{CONFIG}->{PROXY}->{PROXY_PASSWORD}");

   if ($config->{CONFIG}->{BASIC}->{UNHANDLED_ONLY} =~ /true/i) {
      $Win32::GUI::Loft::window{winSettings}->chbSettingsUnhandledOnly->Checked(1);
   } else {
      $Win32::GUI::Loft::window{winSettings}->chbSettingsUnhandledOnly->Checked(0);   
   }


   if ($config->{CONFIG}->{SOUND}->{ENABLE_SOUND} =~ /true/i) {
      $Win32::GUI::Loft::window{winSettings}->chbSettingsEnableSound->Checked(1);
   } else {
      $Win32::GUI::Loft::window{winSettings}->chbSettingsEnableSound->Checked(0);   
   }

   if ($config->{CONFIG}->{SOUND}->{SOUND_CRITONLY} =~ /true/i) {
      $Win32::GUI::Loft::window{winSettings}->chbSettingsSoundCriticalOnly->Checked(1);
   } else {
      $Win32::GUI::Loft::window{winSettings}->chbSettingsSoundCriticalOnly->Checked(0);   
   }

   if ($config->{CONFIG}->{POPUPS}->{ENABLE_POPUPS} =~ /true/i) {
      $Win32::GUI::Loft::window{winSettings}->chbSettingsEnablePopups->Checked(1);
   } else {
      $Win32::GUI::Loft::window{winSettings}->chbSettingsEnablePopups->Checked(0);   
   }

   if ($config->{CONFIG}->{POPUPS}->{POPUPS_CRITONLY} =~ /true/i) {
      $Win32::GUI::Loft::window{winSettings}->chbSettingsPopupsCriticalOnly->Checked(1);
   } else {
      $Win32::GUI::Loft::window{winSettings}->chbSettingsPopupsCriticalOnly->Checked(0);   
   }

   if ($config->{CONFIG}->{POPUPS}->{AUTO_DISMISS} =~ /true/i) {
      $Win32::GUI::Loft::window{winSettings}->chbSettingsDismissPopups->Checked(1);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsPopupDismissTime->ReadOnly(0);
   } else {
      $Win32::GUI::Loft::window{winSettings}->chbSettingsDismissPopups->Checked(0);   
      $Win32::GUI::Loft::window{winSettings}->tfSettingsPopupDismissTime->ReadOnly(1);
   }



   if ($config->{CONFIG}->{AUTHENTICATION}->{AUTH_REQUIRED} =~ /true/i) {
      $Win32::GUI::Loft::window{winSettings}->chbSettingsNagiosAuth->Checked(1);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsNagiosAuthUsername->ReadOnly(0);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsNagiosAuthPassword->ReadOnly(0);
   } else {
      $Win32::GUI::Loft::window{winSettings}->chbSettingsNagiosAuth->Checked(0);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsNagiosAuthUsername->ReadOnly(1);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsNagiosAuthPassword->ReadOnly(1);
   }

   if ($config->{CONFIG}->{PROXY}->{AUTH_REQUIRED} =~ /true/i) {
      $Win32::GUI::Loft::window{winSettings}->chbSettingsUseProxyAuth->Checked(1);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyUsername->ReadOnly(0);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyPassword->ReadOnly(0);
   } else {
      $Win32::GUI::Loft::window{winSettings}->chbSettingsUseProxyAuth->Checked(0);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyUsername->ReadOnly(1);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyPassword->ReadOnly(1);
   }


   if ($config->{CONFIG}->{PROXY}->{PROXY_REQUIRED} =~ /true/i) {
      $Win32::GUI::Loft::window{winSettings}->chbSettingsUseProxy->Checked(1);
      $Win32::GUI::Loft::window{winSettings}->chbSettingsUseProxyAuth->Enable(1);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyAddress->ReadOnly(0);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyPort->ReadOnly(0);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyUsername->ReadOnly(0);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyPassword->ReadOnly(0);
   } else {
      $Win32::GUI::Loft::window{winSettings}->chbSettingsUseProxy->Checked(0);
      $Win32::GUI::Loft::window{winSettings}->chbSettingsUseProxyAuth->Enable(0);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyAddress->ReadOnly(1);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyPort->ReadOnly(1);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyUsername->ReadOnly(1);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyPassword->ReadOnly(1);
  }

   $winSettings->Show();
   1;
}

sub winSettings_Terminate {
   #Store Window location for later use
   print "Hiding winSettings\n";

   $winSettings->Hide();
   0;   
}

sub btnSettingsCancel_Click {
   winSettings_Terminate();
   0;
}

sub btnSettingsApply_Click {
   $config->{CONFIG}->{BASIC}->{CGI_URL} = $Win32::GUI::Loft::window{winSettings}->tfSettingsNagiosCGIUrl->Text();
   $config->{CONFIG}->{BASIC}->{NAGIOS_URL} = $Win32::GUI::Loft::window{winSettings}->tfSettingsNagiosURL->Text();
   $config->{CONFIG}->{BASIC}->{REFRESH_TIMER} = $Win32::GUI::Loft::window{winSettings}->tfSettingsTimeRefresh->Text();
   $config->{CONFIG}->{POPUPS}->{DISMISS_TIMER} = $Win32::GUI::Loft::window{winSettings}->tfSettingsPopupDismissTime->Text();


   $config->{CONFIG}->{AUTHENTICATION}->{AUTH_USERNAME} = $Win32::GUI::Loft::window{winSettings}->tfSettingsNagiosAuthUsername->Text();
   $config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD} = $Win32::GUI::Loft::window{winSettings}->tfSettingsNagiosAuthPassword->Text();
   $config->{CONFIG}->{PROXY}->{PROXY_SERVER} = $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyAddress->Text();
   $config->{CONFIG}->{PROXY}->{PROXY_PORT} = $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyPort->Text();
   $config->{CONFIG}->{PROXY}->{PROXY_USERNAME} = $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyUsername->Text();
   $config->{CONFIG}->{PROXY}->{PROXY_PASSWORD} = $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyPassword->Text();

   if ($Win32::GUI::Loft::window{winSettings}->chbSettingsUnhandledOnly->Checked()) {
      $config->{CONFIG}->{BASIC}->{UNHANDLED_ONLY} = "TRUE";
   } else {
      $config->{CONFIG}->{BASIC}->{UNHANDLED_ONLY} = "FALSE";
   }

   if ($Win32::GUI::Loft::window{winSettings}->chbSettingsEnableSound->Checked()) {
      $config->{CONFIG}->{SOUND}->{ENABLE_SOUND} = "TRUE";
   } else {
      $config->{CONFIG}->{SOUND}->{ENABLE_SOUND} = "FALSE";
   }

   if ($Win32::GUI::Loft::window{winSettings}->chbSettingsSoundCriticalOnly->Checked()) {
      $config->{CONFIG}->{SOUND}->{SOUND_CRITONLY} = "TRUE";
   } else {
      $config->{CONFIG}->{SOUND}->{SOUND_CRITONLY} = "FALSE";
   }

   if ($Win32::GUI::Loft::window{winSettings}->chbSettingsEnablePopups->Checked()) {
      $config->{CONFIG}->{POPUPS}->{ENABLE_POPUPS} = "TRUE";
   } else {
      $config->{CONFIG}->{POPUPS}->{ENABLE_POPUPS} = "FALSE";
   }

   if ($Win32::GUI::Loft::window{winSettings}->chbSettingsPopupsCriticalOnly->Checked()) {
      $config->{CONFIG}->{POPUPS}->{POPUPS_CRITONLY} = "TRUE";
   } else {
      $config->{CONFIG}->{POPUPS}->{POPUPS_CRITONLY} = "FALSE";
   }

   if ($Win32::GUI::Loft::window{winSettings}->chbSettingsDismissPopups->Checked()) {
      $config->{CONFIG}->{POPUPS}->{AUTO_DISMISS} = "TRUE";
   } else {
      $config->{CONFIG}->{POPUPS}->{AUTO_DISMISS} = "FALSE";
   }


   if ($Win32::GUI::Loft::window{winSettings}->chbSettingsNagiosAuth->Checked()) {
      $config->{CONFIG}->{AUTHENTICATION}->{AUTH_REQUIRED} = "TRUE";
   } else {
      $config->{CONFIG}->{AUTHENTICATION}->{AUTH_REQUIRED} = "FALSE";
   }

   if ($Win32::GUI::Loft::window{winSettings}->chbSettingsUseProxyAuth->Checked()) {
      $config->{CONFIG}->{PROXY}->{AUTH_REQUIRED} = "TRUE";
   } else {
      $config->{CONFIG}->{PROXY}->{AUTH_REQUIRED} = "FALSE";
   }

   if ($Win32::GUI::Loft::window{winSettings}->chbSettingsUseProxy->Checked()) {
      $config->{CONFIG}->{PROXY}->{PROXY_REQUIRED} = "TRUE";
   } else {
      $config->{CONFIG}->{PROXY}->{PROXY_REQUIRED} = "FALSE";
   }

   write_configfile("$configfile");
   0;
}

sub chbSettingsNagiosAuth_Click {
   if ($Win32::GUI::Loft::window{winSettings}->chbSettingsNagiosAuth->Checked()) {
      $Win32::GUI::Loft::window{winSettings}->tfSettingsNagiosAuthUsername->ReadOnly(0);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsNagiosAuthPassword->ReadOnly(0);
   } else {
      $Win32::GUI::Loft::window{winSettings}->tfSettingsNagiosAuthUsername->ReadOnly(1);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsNagiosAuthPassword->ReadOnly(1);
   }
   0;
}

sub chbSettingsUseProxyAuth_Click {
   if (($Win32::GUI::Loft::window{winSettings}->chbSettingsUseProxyAuth->Checked()) && ($Win32::GUI::Loft::window{winSettings}->chbSettingsUseProxy->Checked())) {
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyUsername->ReadOnly(0);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyPassword->ReadOnly(0);
   } else {
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyUsername->ReadOnly(1);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyPassword->ReadOnly(1);
   }
   0;
}

sub chbSettingsUseProxy_Click {
   if ($Win32::GUI::Loft::window{winSettings}->chbSettingsUseProxy->Checked()) {
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyAddress->ReadOnly(0);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyPort->ReadOnly(0);
      $Win32::GUI::Loft::window{winSettings}->chbSettingsUseProxyAuth->Enable(1);
      if ($Win32::GUI::Loft::window{winSettings}->chbSettingsUseProxyAuth->Checked()) {
         $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyUsername->ReadOnly(0);
         $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyPassword->ReadOnly(0);
      }
   } else {
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyAddress->ReadOnly(1);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyPort->ReadOnly(1);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyUsername->ReadOnly(1);
      $Win32::GUI::Loft::window{winSettings}->tfSettingsProxyPassword->ReadOnly(1);
      $Win32::GUI::Loft::window{winSettings}->chbSettingsUseProxyAuth->Enable(0);
   }
   0;
}

sub chbSettingsDismissPopups_Click {
   if ($Win32::GUI::Loft::window{winSettings}->chbSettingsDismissPopups->Checked()) {   
      $Win32::GUI::Loft::window{winSettings}->tfSettingsPopupDismissTime->ReadOnly(0);
   } else {
      $Win32::GUI::Loft::window{winSettings}->tfSettingsPopupDismissTime->ReadOnly(1);
   }
}

sub btnSettingsOK_Click {
   btnSettingsApply_Click();
   winSettings_Terminate();
}

sub write_configfile {
   my ($configfile) = @_;
   print "Writing to config file $configfile\n";

   #'Encrypt' the password fields in the config file
   $config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD} = join("\\",   map { ord } (split //,$config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD} ));
   $config->{CONFIG}->{PROXY}->{PROXY_PASSWORD} = join("\\",   map { ord } (split //,$config->{CONFIG}->{PROXY}->{PROXY_PASSWORD}));

   open (OUTFILE,">$configfile");
   print OUTFILE $xml->XMLout($config, keeproot => 1, noattr => 1);
   close OUTFILE;
   
   #'Decrypt' the password fields in the config file
   $config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD} = join("",   map { chr } (split /\\/,$config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD} ));
   $config->{CONFIG}->{PROXY}->{PROXY_PASSWORD} = join("",   map { chr } (split /\\/,$config->{CONFIG}->{PROXY}->{PROXY_PASSWORD}));

}

END {
   print "Terminating Icon $$\n";
   if ($icoTrayicon) {
      $winStatus->AddNotifyIcon(-id   => $$,
                                -name => 'icoTrayicon');
#      Win32::GUI::NotifyIcon->Delete(-id   => $$,
#                                -name => 'icoTrayicon');
   }
   undef $icoTrayicon;
   write_configfile("$configfile");
}

